@@ -17,7 +17,7 @@ use serde::ser::{Serialize, Serializer};
 use xattr::FileExt;
 
 use crate::fileinfo::FileInfo;
-use crate::util::{capitalize, format_date};
+use crate::util::{capitalize, error_exit, format_date};
 use crate::util::format_datetime;
 use crate::util::parse_datetime;
 use crate::util::parse_filesize;
@@ -206,7 +206,7 @@ impl Variant {
                 Ok((dt_from, dt_to)) => {
                     return (dt_from, dt_to);
                 },
-                _ => panic!("Illegal datetime format")
+                _ => error_exit("Can't parse datetime", &self.string_value)
             }
         }
 
@@ -284,6 +284,8 @@ pub enum Function {
     Xattr,
     #[cfg(target_os = "linux")]
     HasCapabilities,
+    #[cfg(target_os = "linux")]
+    HasCapability,
 
     Random,
 }
@@ -323,7 +325,7 @@ impl FromStr for Function {
             "coalesce" => Ok(Function::Coalesce),
             "format_size" | "format_filesize" => Ok(Function::FormatSize),
 
-            "current_date" | "curdate" => Ok(Function::CurrentDate),
+            "current_date" | "cur_date" | "curdate" => Ok(Function::CurrentDate),
             "day" => Ok(Function::Day),
             "month" => Ok(Function::Month),
             "year" => Ok(Function::Year),
@@ -357,6 +359,8 @@ impl FromStr for Function {
             "xattr" => Ok(Function::Xattr),
             #[cfg(target_os = "linux")]
             "has_capabilities" | "has_caps" => Ok(Function::HasCapabilities),
+            #[cfg(target_os = "linux")]
+            "has_capability" | "has_cap" => Ok(Function::HasCapability),
 
             "rand" | "random" => Ok(Function::Random),
 
@@ -422,7 +426,7 @@ impl Function {
         }
 
         #[cfg(target_os = "linux")]
-        if self == &Function::HasCapabilities {
+        if self == &Function::HasCapabilities || self == &Function::HasCapability {
             return true;
         }
 
@@ -715,8 +719,21 @@ pub fn get_value(function: &Option<Function>,
         Some(Function::HasCapabilities) => {
             if let Some(entry) = entry {
                 if let Ok(file) = File::open(&entry.path()) {
-                    if let Ok(xattr) = file.get_xattr("security.capability") {
-                        return Variant::from_bool(xattr.is_some());
+                    if let Ok(caps_xattr) = file.get_xattr("security.capability") {
+                        return Variant::from_bool(caps_xattr.is_some());
+                    }
+                }
+            }
+
+            return Variant::empty(VariantType::Bool);
+        }
+        #[cfg(target_os = "linux")]
+        Some(Function::HasCapability) => {
+            if let Some(entry) = entry {
+                if let Ok(file) = File::open(&entry.path()) {
+                    if let Ok(Some(caps_xattr)) = file.get_xattr("security.capability") {
+                        let caps_string = crate::util::capabilities::parse_capabilities(caps_xattr);
+                        return Variant::from_bool(caps_string.contains(&function_arg));
                     }
                 }
             }
@@ -738,11 +755,11 @@ pub fn get_value(function: &Option<Function>,
                         let limit = function_args.get(0).unwrap();
                         match limit.parse::<i64>() {
                             Ok(limit) => return Variant::from_int(rng.gen_range(val..limit)),
-                            _ => panic!("Could not parse function argument")
+                            _ => error_exit("Could not parse limit argument of RANDOM function", limit.as_str())
                         }
                     }
                 },
-                _ => panic!("Could not parse function argument")
+                _ => error_exit("Could not parse an argument of RANDOM function", function_arg.as_str())
             }
         },
         _ => {
