@@ -15,10 +15,13 @@ pub enum Lexem {
     String(String),
     Open,
     Close,
+    CurlyOpen,
+    CurlyClose,
     ArithmeticOperator(String),
     And,
     Or,
     Not,
+    Group,
     Order,
     By,
     DescendingOrder,
@@ -87,7 +90,7 @@ impl<'a> Lexer<'a> {
                             if maybe_expr {
                                 break;
                             }
-                        } else if c == ' ' || c == ',' || c == '(' || c == ')' || self.is_op_char(c) {
+                        } else if c == ' ' || c == ',' || is_paren_char(c) || self.is_op_char(c) {
                             break
                         }
                     }
@@ -101,8 +104,8 @@ impl<'a> Lexer<'a> {
                         ' ' => {},
                         '\'' => mode = LexingMode::String,
                         ',' => mode = LexingMode::Comma,
-                        '(' => mode = LexingMode::Open,
-                        ')' => mode = LexingMode::Close,
+                        '(' | '{' => { s.push(c); mode = LexingMode::Open },
+                        ')' | '}' => { s.push(c); mode = LexingMode::Close },
                         _ => {
                             mode = if self.is_op_char(c) {
                                 LexingMode::Operator
@@ -129,15 +132,18 @@ impl<'a> Lexer<'a> {
             LexingMode::Operator => Some(Lexem::Operator(s)),
             LexingMode::ArithmeticOperator => Some(Lexem::ArithmeticOperator(s)),
             LexingMode::Comma => Some(Lexem::Comma),
-            LexingMode::Open => Some(Lexem::Open),
-            LexingMode::Close => Some(Lexem::Close),
+            LexingMode::Open if &s == "(" => { s.clear(); Some(Lexem::Open) },
+            LexingMode::Open if &s == "{" => { s.clear(); Some(Lexem::CurlyOpen) },
+            LexingMode::Close if &s == ")" => { s.clear(); Some(Lexem::Close) },
+            LexingMode::Close if &s == "}" => { s.clear(); Some(Lexem::CurlyClose) },
             LexingMode::RawString => {
                 match s.to_lowercase().as_str() {
-                    "from" => { self.before_from = false; Some(Lexem::From) },
+                    "from" => { self.before_from = false; self.after_where = false; Some(Lexem::From) },
                     "where" => { self.after_where = true; Some(Lexem::Where) },
                     "or" => Some(Lexem::Or),
                     "and" => Some(Lexem::And),
                     "not" if self.after_where => Some(Lexem::Not),
+                    "group" => Some(Lexem::Group),
                     "order" => Some(Lexem::Order),
                     "by" => Some(Lexem::By),
                     "asc" => self.next_lexem(),
@@ -163,8 +169,8 @@ impl<'a> Lexer<'a> {
 
     fn is_arithmetic_op_char(&self, c: char) -> bool {
         match c {
-            '+' | '-' => self.before_from,
-            '*' | '/' | '%' => self.before_from && !self.after_open && !self.after_operator,
+            '+' | '-' => self.before_from || self.after_where,
+            '*' | '/' | '%' => (self.before_from || self.after_where) && !self.after_open && !self.after_operator,
             _ => false
         }
     }
@@ -181,6 +187,9 @@ impl<'a> Lexer<'a> {
     }
 }
 
+fn is_paren_char(c: char) -> bool {
+    c == '(' || c == ')' || c == '{' || c == '}'
+}
 
 lazy_static! {
     static ref DATE_ALIKE_REGEX: Regex = Regex::new("(\\d{4})-?(\\d{2})?").unwrap();
@@ -556,4 +565,20 @@ mod tests {
         assert_eq!(lexer.next_lexem(), Some(Lexem::From));
         assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("/test"))));
     }
+
+    #[test]
+    fn group_by() {
+        let mut lexer = Lexer::new("select AVG(size) from /test group by mime");
+
+        assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("select"))));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("AVG"))));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::Open));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("size"))));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::Close));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::From));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("/test"))));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::Group));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::By));
+        assert_eq!(lexer.next_lexem(), Some(Lexem::RawString(String::from("mime"))));
+    }
 }
