@@ -131,7 +131,7 @@ fn main() -> ExitCode {
                     match readline {
                         Ok(query) => {
                             let _ = rl.add_history_entry(query.as_str());
-                            exec_search(query, &config, no_color);
+                            exec_search(query, &mut config, no_color);
                         },
                         Err(ReadlineError::Interrupted) => {
                             println!("CTRL-C");
@@ -156,7 +156,7 @@ fn main() -> ExitCode {
         }
     } else {
         let query = args.join(" ");
-        exit_value = Some(exec_search(query, &config, no_color));
+        exit_value = Some(exec_search(query, &mut config, no_color));
     }
 
     config.save();
@@ -168,7 +168,7 @@ fn main() -> ExitCode {
     ExitCode::SUCCESS
 }
 
-fn exec_search(query: String, config: &Config, no_color: bool) -> u8 {
+fn exec_search(query: String, config: &mut Config, no_color: bool) -> u8 {
     let mut p = Parser::new();
     let query = p.parse(&query, config.debug);
 
@@ -181,7 +181,7 @@ fn exec_search(query: String, config: &Config, no_color: bool) -> u8 {
             let is_terminal = atty::is(Stream::Stdout);
             let use_colors = !no_color && is_terminal;
 
-            let mut searcher = Searcher::new(query, config.clone(), use_colors);
+            let mut searcher = Searcher::new(&query, config, use_colors);
             searcher.list_search_results().unwrap();
 
             let error_count = searcher.error_count;
@@ -214,7 +214,7 @@ fn short_usage_info(no_color: bool) {
     }
 
     println!();
-    println!("Usage: fselect [ARGS] COLUMN[, COLUMN...] [from PATH[, PATH...]] [where EXPR] [order by COLUMN (asc|desc), ...] [limit N] [into FORMAT]");
+    println!("Usage: fselect [ARGS] COLUMN[, COLUMN...] [from PATH[, PATH...]] [where EXPR] [group by COLUMN, ...] [order by COLUMN (asc|desc), ...] [limit N] [into FORMAT]");
 }
 
 fn help_hint() {
@@ -285,6 +285,7 @@ Column Options:
     is_socket                       Returns a boolean signifying whether the file path is a socket file
     is_hidden                       Returns a boolean signifying whether the file is a hidden file (e.g., files that start with a dot on *nix)
     has_xattrs                      Returns a boolean signifying whether the file has extended attributes
+    capabilities | caps             Returns a string describing Linux capabilities assigned to a file
 
     device (Linux only)             Returns the code of device the file is stored on
     inode (Linux only)              Returns the number of inode
@@ -366,7 +367,8 @@ Functions:
         VAR_POP | VARIANCE          Population variance
         VAR_SAMP                    Sample variance
     Date:
-        CURRENT_DATE | CURDATE      Returns current date
+        CURRENT_DATE | CUR_DATE |
+        CURDATE                     Returns current date
         DAY                         Returns day of the month
         MONTH                       Returns month of the year
         YEAR                        Returns year of the date
@@ -379,7 +381,8 @@ Functions:
     Xattr:
         HAS_XATTR                   Used to check if xattr exists (unix-only)
         XATTR                       Returns value of xattr (unix-only)
-        HAS_CAPABILITIES | HAS_CAPS Check if Linux capability exists for the file
+        HAS_CAPABILITIES | HAS_CAPS Check if any Linux capability exists for the file
+        HAS_CAPABILITY or HAS_CAP   Check if given Linux capability exists for the file
     String:
         LENGTH | LEN                Returns length of string value
         LOWER | LOWERCASE | LCASE   Returns lowercase value
