@@ -19,13 +19,17 @@ use directories::UserDirs;
 pub struct Parser {
     lexems: Vec<Lexem>,
     index: usize,
+    roots_parsed: bool,
+    where_parsed: bool,
 }
 
 impl Parser {
     pub fn new() -> Parser {
         Parser {
             lexems: vec![],
-            index: 0
+            index: 0,
+            roots_parsed: false,
+            where_parsed: false,
         }
     }
 
@@ -41,7 +45,10 @@ impl Parser {
 
         let fields = self.parse_fields()?;
         let mut roots = self.parse_roots();
+        self.roots_parsed = true;
         let expr = self.parse_where()?;
+        self.where_parsed = true;
+        let grouping_fields = self.parse_group_by()?;
         let (ordering_fields, ordering_asc) = self.parse_order_by(&fields)?;
         let mut limit = self.parse_limit()?;
         let output_format = self.parse_output_format()?;
@@ -71,6 +78,7 @@ impl Parser {
             fields,
             roots,
             expr,
+            grouping_fields: Rc::new(grouping_fields),
             ordering_fields: Rc::new(ordering_fields),
             ordering_asc: Rc::new(ordering_asc),
             limit,
@@ -419,12 +427,12 @@ impl Parser {
 
         if let Ok(Some(expr)) = result.clone() {
             if let Some(field) = expr.field {
-                if expr.left.is_none() && expr.right.is_none() && field.is_boolean_field() {
+                if expr.left.is_none() && expr.right.is_none() && field.is_boolean_field() && self.roots_parsed && !self.where_parsed {
                     result = Ok(Some(Expr::op(Expr::field(field), Op::Eq, Expr::value(String::from("true")))));
                 }
             } else if let Some(function) = expr.function {
-                if expr.left.is_some() && expr.right.is_none() && (expr.args.is_none() || expr.args.unwrap().is_empty()) && function.is_boolean_function() {
-                    let func_expr = Expr::function_left(function, *expr.left.unwrap());
+                if expr.right.is_none() && (expr.args.is_none() || expr.args.unwrap().is_empty()) && function.is_boolean_function() && self.roots_parsed && !self.where_parsed {
+                    let func_expr = Expr::function_left(function, expr.left);
                     result = Ok(Some(Expr::op(func_expr, Op::Eq, Expr::value(String::from("true")))));
                 }
             }
@@ -578,11 +586,16 @@ impl Parser {
     }
 
     fn parse_function(&mut self, function: Function) -> Result<Expr, String> {
+        let is_boolean_function = function.is_boolean_function();
         let mut function_expr = Expr::function(function);
 
         let mut curly_mode = false;
         if let Some(lexem) = self.next_lexem() {
             if lexem != Lexem::Open && lexem != Lexem::CurlyOpen {
+                if is_boolean_function {
+                   return Ok(function_expr);
+                }
+
                 return Err("Error in function expression".to_string());
             }
 
@@ -620,6 +633,35 @@ impl Parser {
         }
     }
 
+    fn parse_group_by(&mut self) -> Result<Vec<Expr>, String> {
+        let mut group_by_fields: Vec<Expr> = vec![];
+
+        if let Some(Lexem::Group) = self.next_lexem() {
+            if let Some(Lexem::By) = self.next_lexem() {
+                loop {
+                    match self.next_lexem() {
+                        Some(Lexem::Comma) => {},
+                        Some(Lexem::RawString(_)) => {
+                            self.drop_lexem();
+                            let group_field = self.parse_expr().unwrap().unwrap();
+                            group_by_fields.push(group_field);
+                        },
+                        _ => {
+                            self.drop_lexem();
+                            break;
+                        },
+                    }
+                }
+            } else {
+                self.drop_lexem();
+            }
+        } else {
+            self.drop_lexem();
+        }
+
+        Ok(group_by_fields)
+    }
+
     fn parse_order_by(&mut self, fields: &Vec<Expr>) -> Result<(Vec<Expr>, Vec<bool>), String> {
         let mut order_by_fields: Vec<Expr> = vec![];
         let mut order_by_directions: Vec<bool> = vec![];
@@ -944,6 +986,33 @@ mod tests {
         assert_eq!(query.expr, query2.expr);
     }
 
+    #[test]
+    #[cfg(target_os = "linux")]
+    fn simple_function_without_args_syntax_in_where() {
+        let query = "select name, caps from /home/user where HAS_CAPS()";
+        let mut p = Parser::new();
+        let query = p.parse(&query, false).unwrap();
+
+        let query2 = "select name, caps from /home/user where HAS_CAPS";
+        let mut p2 = Parser::new();
+        let query2 = p2.parse(&query2, false).unwrap();
+
+        assert_eq!(query.expr, query2.expr);
+    }
+
+    #[test]
+    fn simple_function_without_args_syntax() {
+        let query = "select CURDATE()";
+        let mut p = Parser::new();
+        let query = p.parse(&query, false).unwrap();
+
+        let query2 = "select CURDATE";
+        let mut p2 = Parser::new();
+        let query2 = p2.parse(&query2, false).unwrap();
+
+        assert_eq!(query.expr, query2.expr);
+    }
+
     #[test]
     fn from_at_the_end_of_the_query() {
         let query = "select name where not name like '%.tmp' from /test gitignore mindepth 2";
@@ -973,4 +1042,19 @@ mod tests {
 
         assert_eq!(query.expr, query2.expr);
     }
+
+    #[test]
+    fn query_with_group_by() {
+        let query = "select AVG(size) from /test group by mime";
+        let mut p = Parser::new();
+        let query = p.parse(&query, false).unwrap();
+
+        assert_eq!(query.fields, vec![Expr::function_left(Function::Avg, Some(Box::new(Expr::field(Field::Size))))]);
+
+        assert_eq!(query.roots, vec![
+            Root::new(String::from("/test"), 0, 0, false, false, None, None, None, Bfs, false),
+        ]);
+
+        assert_eq!(query.grouping_fields, Rc::new(vec![Expr::field(Field::Mime)]));
+    }
 }
