@@ -34,15 +34,12 @@ use crate::fileinfo::to_file_info;
 use crate::function;
 use crate::function::Variant;
 use crate::function::VariantType;
-use crate::ignore::docker::DockerignoreFilter;
+use crate::ignore::docker::{DockerignoreFilter, search_upstream_dockerignore};
 use crate::ignore::docker::matches_dockerignore_filter;
-use crate::ignore::docker::parse_dockerignore;
-use crate::ignore::git::GitignoreFilter;
+use crate::ignore::git::{get_gitignore_filters, GitignoreFilter, search_upstream_gitignore, update_gitignore_map};
 use crate::ignore::git::matches_gitignore_filter;
-use crate::ignore::git::parse_gitignore;
-use crate::ignore::hg::HgignoreFilter;
+use crate::ignore::hg::{HgignoreFilter, search_upstream_hgignore};
 use crate::ignore::hg::matches_hgignore_filter;
-use crate::ignore::hg::parse_hgignore;
 use crate::mode;
 use crate::operators::LogicalOp;
 use crate::operators::Op;
@@ -52,9 +49,9 @@ use crate::util::*;
 use crate::util::dimensions::get_dimensions;
 use crate::output::ResultsWriter;
 
-pub struct Searcher {
-    query: Query,
-    config : Config,
+pub struct Searcher<'a> {
+    query: &'a Query,
+    config : &'a Config,
     use_colors: bool,
     results_writer: ResultsWriter,
     #[cfg(all(unix, feature = "users"))]
@@ -62,6 +59,7 @@ pub struct Searcher {
     regex_cache: HashMap<String, Regex>,
     found: u32,
     raw_output_buffer: Vec<HashMap<String, String>>,
+    partitioned_output_buffer: HashMap<Vec<String>, Vec<HashMap<String, String>>>,
     output_buffer: TopN<Criteria<String>, String>,
     gitignore_map: HashMap<PathBuf, Vec<GitignoreFilter>>,
     hgignore_filters: Vec<HgignoreFilter>,
@@ -91,8 +89,8 @@ pub struct Searcher {
     pub error_count: i32,
 }
 
-impl Searcher {
-    pub fn new(query: Query, config: Config, use_colors: bool) -> Self {
+impl <'a> Searcher<'a> {
+    pub fn new(query: &'a Query, config: &'a Config, use_colors: bool) -> Self {
         let limit = query.limit;
 
         let results_writer = ResultsWriter::new(&query.output_format);
@@ -106,6 +104,7 @@ impl Searcher {
             regex_cache: HashMap::new(),
             found: 0,
             raw_output_buffer: vec![],
+            partitioned_output_buffer: HashMap::new(),
             output_buffer: if limit == 0 { TopN::limitless() } else { TopN::new(limit) },
             gitignore_map: HashMap::new(),
             hgignore_filters: vec![],
@@ -181,7 +180,7 @@ impl Searcher {
                 let mut ext_roots: Vec<String> = vec![];
                 let parts = root.path.split('/').collect::<Vec<&str>>();
                 for part in parts {
-                    if self.looks_like_regexp(part) {
+                    if looks_like_regexp(part) {
                         let rx_string = format!("^{}$", part);
                         let rx = Regex::new(&rx_string).unwrap();
                         let mut tmp = vec![];
@@ -250,7 +249,7 @@ impl Searcher {
             }
         }
 
-        for root in roots.clone() {
+        for root in roots {
             self.current_follow_symlinks = root.symlinks;
 
             let root_dir = Path::new(&root.path);
@@ -263,15 +262,15 @@ impl Searcher {
             let traversal_mode = root.traversal;
 
             if apply_gitignore {
-                self.search_upstream_gitignore(&root_dir);
+                search_upstream_gitignore(&mut self.gitignore_map, &root_dir);
             }
 
             if apply_hgignore {
-                self.search_upstream_hgignore(&root_dir);
+                search_upstream_hgignore(&mut self.hgignore_filters, &root_dir);
             }
 
             if apply_dockerignore {
-                self.search_upstream_dockerignore(&root_dir);
+                search_upstream_dockerignore(&mut self.dockerignore_filters, &root_dir);
             }
 
             self.dir_queue.clear();
@@ -302,21 +301,49 @@ impl Searcher {
         }
 
         if self.has_aggregate_column() {
+            if !self.query.grouping_fields.is_empty() {
+                if self.partitioned_output_buffer.is_empty() {
+                    self.partitioned_output_buffer = self.partition_output_buffer();
+                }
 
-            let mut buf = WritableBuffer::new();
-            let mut items: Vec<(String, String)> = Vec::new();
+                let group_keys: Vec<String> = self.query.grouping_fields.iter().map(|f| f.to_string()).collect();
+                let buffer_partitions = self.partitioned_output_buffer.clone();
 
-            for column_expr in &self.query.fields.clone() {
-                let record = format!("{}", self.get_column_expr_value(None, &None, &mut HashMap::new(), column_expr));
-                let field_name = column_expr.to_string().to_lowercase();
-                items.push((field_name, record));
-            }
+                buffer_partitions.iter().for_each(|f| {
+                    let mut buf = WritableBuffer::new();
+                    let mut items: Vec<(String, String)> = Vec::new();
 
-            self.results_writer.write_row(&mut buf, items)?;
+                    let mut file_map = HashMap::new();
+                    for (i, k) in group_keys.clone().into_iter().enumerate() {
+                        file_map.insert(k, f.0.get(i).unwrap().clone());
+                    }
 
-            if let Err(e) = write!(std::io::stdout(), "{}", String::from(buf)) {
-                if e.kind() == ErrorKind::BrokenPipe {
-                    return Ok(());
+                    for column_expr in &self.query.fields.clone() {
+                        let record = format!("{}", self.get_column_expr_value(None, &None, &mut file_map, Some(f.1), column_expr));
+                        let field_name = column_expr.to_string().to_lowercase();
+                        items.push((field_name, record));
+                    }
+
+                    let _ = self.results_writer.write_row(&mut buf, items);
+
+                    let _ = write!(std::io::stdout(), "{}", String::from(buf));
+                })
+            } else {
+                let mut buf = WritableBuffer::new();
+                let mut items: Vec<(String, String)> = Vec::new();
+
+                for column_expr in &self.query.fields.clone() {
+                    let record = format!("{}", self.get_column_expr_value(None, &None, &mut HashMap::new(), None, column_expr));
+                    let field_name = column_expr.to_string().to_lowercase();
+                    items.push((field_name, record));
+                }
+
+                self.results_writer.write_row(&mut buf, items)?;
+
+                if let Err(e) = write!(std::io::stdout(), "{}", String::from(buf)) {
+                    if e.kind() == ErrorKind::BrokenPipe {
+                        return Ok(());
+                    }
                 }
             }
         } else if self.is_buffered() {
@@ -344,139 +371,6 @@ impl Searcher {
         Ok(())
     }
 
-    fn search_upstream_gitignore(&mut self, dir: &Path) {
-        if let Ok(canonical_path) = crate::util::canonical_path(&dir.to_path_buf()) {
-            let mut path = PathBuf::from(canonical_path);
-
-            loop {
-                let parent_found = path.pop();
-
-                if !parent_found {
-                    return;
-                }
-
-                self.update_gitignore_map(&mut path);
-            }
-        }
-    }
-
-    fn update_gitignore_map(&mut self, path: &Path) {
-        let gitignore_file = path.join(".gitignore");
-        if gitignore_file.is_file() {
-            let regexes = parse_gitignore(&gitignore_file, &path);
-            self.gitignore_map.insert(path.to_path_buf(), regexes);
-        }
-    }
-
-    fn get_gitignore_filters(&self, dir: &Path) -> Vec<GitignoreFilter> {
-        let mut result = vec![];
-
-        for (dir_path, regexes) in &self.gitignore_map {
-            if dir.to_path_buf() == *dir_path {
-                for ref mut rx in regexes {
-                    result.push(rx.clone());
-                }
-
-                return result;
-            }
-        }
-
-        let mut path = dir.to_path_buf();
-
-        loop {
-            let parent_found = path.pop();
-
-            if !parent_found {
-                return result;
-            }
-
-            for (dir_path, regexes) in &self.gitignore_map {
-                if path == *dir_path {
-                    let mut tmp = vec![];
-                    for ref mut rx in regexes {
-                        tmp.push(rx.clone());
-                    }
-                    tmp.append(&mut result);
-                    result.clear();
-                    result.append(&mut tmp);
-                }
-            }
-        }
-    }
-
-    fn search_upstream_hgignore(&mut self, dir: &Path) {
-        if let Ok(canonical_path) = crate::util::canonical_path(&dir.to_path_buf()) {
-            let mut path = PathBuf::from(canonical_path);
-
-            loop {
-                let hgignore_file = path.clone().join(".hgignore");
-                let hg_directory = path.clone().join(".hg");
-
-                if hgignore_file.is_file() && hg_directory.is_dir() {
-                    self.update_hgignore_filters(&mut path);
-                    return;
-                }
-
-                let parent_found = path.pop();
-
-                if !parent_found {
-                    return;
-                }
-            }
-        }
-    }
-
-    fn update_hgignore_filters(&mut self, path: &Path) {
-        let hgignore_file = path.join(".hgignore");
-        if hgignore_file.is_file() {
-            let regexes = parse_hgignore(&hgignore_file, &path);
-            match regexes {
-                Ok(ref regexes) => {
-                    self.hgignore_filters.append(&mut regexes.clone());
-                },
-                Err(err) => {
-                    eprintln!("{}: {}", path.to_string_lossy(), err);
-                }
-            }
-        }
-    }
-
-    fn search_upstream_dockerignore(&mut self, dir: &Path) {
-        if let Ok(canonical_path) = crate::util::canonical_path(&dir.to_path_buf()) {
-            let mut path = PathBuf::from(canonical_path);
-
-            loop {
-                let dockerignore_file = path.join(".dockerignore");
-
-                if dockerignore_file.is_file() {
-                    self.update_dockerignore_filters(&mut path);
-                    return;
-                }
-
-                let parent_found = path.pop();
-
-                if !parent_found {
-                    return;
-                }
-            }
-        }
-    }
-
-    fn update_dockerignore_filters(&mut self, path: &Path) {
-        let dockerignore_file = path.join(".dockerignore");
-        if dockerignore_file.is_file() {
-            let regexes = parse_dockerignore(&dockerignore_file, &path);
-            match regexes {
-                Ok(ref regexes) => {
-                    self.dockerignore_filters.append(&mut regexes.clone());
-                },
-                Err(err) => {
-                    eprintln!("{}: {}", path.to_string_lossy(), err);
-                }
-            }
-        }
-    }
-
     fn visit_dir(&mut self,
                  dir: &Path,
                  min_depth: u32,
@@ -518,8 +412,8 @@ impl Searcher {
 
         if apply_gitignore {
             let canonical_path = PathBuf::from(canonical_path);
-            self.update_gitignore_map(&canonical_path);
-            gitignore_filters = Some(self.get_gitignore_filters(&canonical_path));
+            update_gitignore_map(&mut self.gitignore_map, &canonical_path);
+            gitignore_filters = Some(get_gitignore_filters(&mut self.gitignore_map, &canonical_path));
         }
 
         match fs::read_dir(dir) {
@@ -682,9 +576,10 @@ impl Searcher {
                              entry: Option<&DirEntry>,
                              file_info: &Option<FileInfo>,
                              file_map: &mut HashMap<String, String>,
+                             buffer_data: Option<&Vec<HashMap<String, String>>>,
                              column_expr: &Expr) -> Variant {
         if let Some(ref _function) = column_expr.function {
-            let result = self.get_function_value(entry, file_info, file_map, column_expr);
+            let result = self.get_function_value(entry, file_info, file_map, buffer_data, column_expr);
             file_map.insert(column_expr.to_string(), result.to_string());
             return result;
         }
@@ -710,11 +605,11 @@ impl Searcher {
         let result;
 
         if let Some(ref left) = column_expr.left {
-            let left_result = self.get_column_expr_value(entry, file_info, file_map, left);
+            let left_result = self.get_column_expr_value(entry, file_info, file_map, buffer_data, left);
 
             if let Some(ref op) = column_expr.arithmetic_op {
                 if let Some(ref right) = column_expr.right {
-                    let right_result = self.get_column_expr_value(entry, file_info, file_map, right);
+                    let right_result = self.get_column_expr_value(entry, file_info, file_map, buffer_data, right);
                     result = op.calc(&left_result, &right_result);
                     file_map.insert(column_expr.to_string(), result.to_string());
                 } else {
@@ -734,6 +629,7 @@ impl Searcher {
                           entry: Option<&DirEntry>,
                           file_info: &Option<FileInfo>,
                           file_map: &mut HashMap<String, String>,
+                          buffer_data: Option<&Vec<HashMap<String, String>>>,
                           column_expr: &Expr) -> Variant {
         let dummy = Expr::value(String::from(""));
         let boxed_dummy = &Box::from(dummy);
@@ -746,20 +642,23 @@ impl Searcher {
         let function = &column_expr.function.as_ref().unwrap();
 
         if function.is_aggregate_function() {
-            let _ = self.get_column_expr_value(entry, file_info, file_map, left_expr);
+            if entry.is_some() {
+                return Variant::empty(VariantType::Float);
+            }
+
+            let _ = self.get_column_expr_value(entry, file_info, file_map, buffer_data, left_expr);
             let buffer_key = left_expr.to_string();
             let aggr_result = function::get_aggregate_value(&column_expr.function,
-                                                            &self.raw_output_buffer,
-                                                            //left_expr.field.as_ref().unwrap().to_string().to_lowercase(),
+                                                            buffer_data.unwrap_or(&self.raw_output_buffer),
                                                             buffer_key,
                                                             &column_expr.val);
             return Variant::from_string(&aggr_result);
         } else {
-            let function_arg = self.get_column_expr_value(entry, file_info, file_map, left_expr);
+            let function_arg = self.get_column_expr_value(entry, file_info, file_map, buffer_data, left_expr);
             let mut function_args = vec![];
             if let Some(args) = &column_expr.args {
                 for arg in args {
-                    let arg_value = self.get_column_expr_value(entry, file_info, file_map, arg);
+                    let arg_value = self.get_column_expr_value(entry, file_info, file_map, buffer_data, arg);
                     function_args.push(arg_value.to_string());
                 }
             }
@@ -770,6 +669,22 @@ impl Searcher {
         }
     }
 
+    fn partition_output_buffer(&self) -> HashMap<Vec<String>, Vec<HashMap<String, String>>> {
+        let group_fields: Vec<String> = self.query.grouping_fields.iter().map(|ref expr| expr.to_string()).collect();
+        let mut result: HashMap<Vec<String>, Vec<HashMap<String, String>>> = HashMap::new();
+
+        self.raw_output_buffer.iter().for_each(|item| {
+            let key: Vec<String> = group_fields.iter().map(|f| item.get(f).unwrap_or(&String::new()).clone()).collect();
+            if result.contains_key(&key) {
+                result.get_mut(&key).unwrap().push(item.clone());
+            } else {
+                result.insert(key, vec![item.clone()]);
+            }
+        });
+
+        result
+    }
+
     fn update_file_metadata(&mut self, entry: &DirEntry) {
         if !self.file_metadata_set {
             self.file_metadata_set = true;
@@ -1171,6 +1086,19 @@ impl Searcher {
                         return Variant::from_bool(false);
                     }
             },
+            Field::Capabilities => {
+                #[cfg(target_os = "linux")]
+                    {
+                        if let Ok(file) = fs::File::open(&entry.path()) {
+                            if let Ok(Some(caps_xattr)) = file.get_xattr("security.capability") {
+                                let caps_string = crate::util::capabilities::parse_capabilities(caps_xattr);
+                                return Variant::from_string(&caps_string);
+                            }
+                        }
+                    }
+
+                return Variant::empty(VariantType::String);
+            },
             Field::IsShebang => {
                 return Variant::from_bool(is_shebang(&entry.path()));
             },
@@ -1478,7 +1406,7 @@ impl Searcher {
                   file_info: &Option<FileInfo>) -> std::io::Result<bool> {
         self.clear_file_data();
 
-        if let Some(ref expr) = self.query.expr.clone() {
+        if let Some(ref expr) = self.query.expr {
             let result = self.conforms(entry, file_info, expr);
             if !result {
                 return Ok(true);
@@ -1504,7 +1432,7 @@ impl Searcher {
         let mut items: Vec<(String, String)> = Vec::new();
 
         for field in self.query.fields.clone().iter() {
-            let record = self.get_column_expr_value(Some(entry), file_info, &mut file_map, &field);
+            let record = self.get_column_expr_value(Some(entry), file_info, &mut file_map, None, &field);
 
             let value = match self.use_colors && field.contains_colorized() {
                 true => self.colorize(&record.to_string()),
@@ -1513,10 +1441,19 @@ impl Searcher {
             items.push((field.to_string(), value));
         }
 
+        for field in self.query.grouping_fields.iter() {
+            match file_map.get(&field.to_string()) {
+                None => {
+                    self.get_column_expr_value(Some(entry), file_info, &mut file_map, None, &field);
+                },
+                _ => {}
+            }
+        }
+
         for (idx, field) in self.query.ordering_fields.clone().iter().enumerate() {
             criteria[idx] = match file_map.get(&field.to_string()) {
                 Some(record) => record.clone(),
-                None => self.get_column_expr_value(Some(entry), file_info, &mut file_map, &field).to_string()
+                None => self.get_column_expr_value(Some(entry), file_info, &mut file_map, None, &field).to_string()
             }
         }
 
@@ -1548,7 +1485,7 @@ impl Searcher {
             style = self.lscolors.style_for_path(Path::new(&value));
         }
 
-        let ansi_style = style.map(Style::to_ansi_term_style).unwrap_or_default();
+        let ansi_style = style.map(Style::to_nu_ansi_term_style).unwrap_or_default();
 
         format!("{}", ansi_style.paint(value))
     }
@@ -1618,8 +1555,8 @@ impl Searcher {
                 }
             }
         } else if let Some(ref op) = expr.op {
-            let field_value = self.get_column_expr_value(Some(entry), file_info, &mut HashMap::new(), expr.left.as_ref().unwrap());
-            let value = self.get_column_expr_value(Some(entry), file_info, &mut HashMap::new(), expr.right.as_ref().unwrap());
+            let field_value = self.get_column_expr_value(Some(entry), file_info, &mut HashMap::new(), None, expr.left.as_ref().unwrap());
+            let value = self.get_column_expr_value(Some(entry), file_info, &mut HashMap::new(), None, expr.right.as_ref().unwrap());
 
             result = match field_value.get_type() {
                 VariantType::String => {
@@ -1690,9 +1627,7 @@ impl Searcher {
                                             self.regex_cache.insert(val, regex.clone());
                                             return regex.is_match(&field_value.to_string());
                                         },
-                                        _ => {
-                                            panic!("Incorrect regex expression")
-                                        }
+                                        _ => error_exit("Incorrect regex expression", val.as_str())
                                     }
                                 }
                             }
@@ -1710,9 +1645,7 @@ impl Searcher {
                                             self.regex_cache.insert(val, regex.clone());
                                             return !regex.is_match(&field_value.to_string());
                                         },
-                                        _ => {
-                                            panic!("Incorrect regex expression")
-                                        }
+                                        _ => error_exit("Incorrect regex expression", val.as_str())
                                     }
                                 }
                             }
@@ -1731,9 +1664,7 @@ impl Searcher {
                                             self.regex_cache.insert(val, regex.clone());
                                             return regex.is_match(&field_value.to_string());
                                         },
-                                        _ => {
-                                            panic!("Incorrect LIKE expression")
-                                        }
+                                        _ => error_exit("Incorrect LIKE expression", val.as_str())
                                     }
                                 }
                             }
@@ -1752,9 +1683,7 @@ impl Searcher {
                                             self.regex_cache.insert(val, regex.clone());
                                             return !regex.is_match(&field_value.to_string());
                                         },
-                                        _ => {
-                                            panic!("Incorrect LIKE expression")
-                                        }
+                                        _ => error_exit("Incorrect LIKE expression", val.as_str())
                                     }
                                 }
                             }
@@ -1856,8 +1785,4 @@ impl Searcher {
     fn is_video(&self, file_name: &str) -> bool {
         has_extension(file_name, &self.config.is_video)
     }
-
-    fn looks_like_regexp(&self, s: &str) -> bool {
-        s.contains('*') || s.contains('[') || s.contains('?')
-    }
 }
